# Copyright (C) 2002,2004 Stichting LogReport Foundation logreport@logreport.org

# This file is part of Lire.

# Lire is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with this program (see COPYING); if not, check with
# http://www.gnu.org/copyleft/gpl.html.

# Authors:
#   E.L. Willighagen <egonw@logreport.org>
#   Francis J. Lacoste <flacoste@logreport.org>

package Lire::DlfAnalysers::AttackCategoriser;

use strict;

use base qw/Lire::DlfCategoriser/;

use Lire::WWW::Filename;

sub new {
    return bless {}, shift;
}

sub name {
    return "www-attack";
}

sub title {
    return "Attack DlfAnalyser";
}

sub description {
    return '<para>This categoriser detects some common <quote>attacks</quote> based on the request.</para>';
}

sub src_schema {
    return "www";
}

sub dst_schema {
    return "www-attack";
}

sub initialise {
    my ( $self, $config ) = @_;

    $self->{'analyser'} = new Lire::WWW::Filename( 'default' => 'LIRE_UNDEF' );

    return;
}

sub categorise {
    my ( $self, $dlf ) = @_;

    return unless defined $dlf->{'requested_page'};
    $self->{'analyser'}->setFilename( $dlf->{ 'requested_page'} );

    $dlf->{'attack'} = $self->{'analyser'}->getAttack();
    $dlf->{'attack'} = undef if $dlf->{'attack'} eq 'LIRE_UNDEF';

    return;
}

# keep perl happy
1;
